@@ -18,6 +18,7 @@ import enum
 import itertools
 
 from google.cloud.bigquery_v2 import types as gapic_types
+from google.cloud.bigquery.query import ScalarQueryParameterType
 
 
 class Compression(object):
@@ -202,8 +203,8 @@ class SqlTypeNames(str, enum.Enum):
     INT64 = "INTEGER"
     FLOAT = "FLOAT"
     FLOAT64 = "FLOAT"
-    NUMERIC = "NUMERIC"
-    BIGNUMERIC = "BIGNUMERIC"
+    DECIMAL = NUMERIC = "NUMERIC"
+    BIGDECIMAL = BIGNUMERIC = "BIGNUMERIC"
     BOOLEAN = "BOOLEAN"
     BOOL = "BOOLEAN"
     GEOGRAPHY = "GEOGRAPHY"  # NOTE: not available in legacy types
@@ -215,6 +216,28 @@ class SqlTypeNames(str, enum.Enum):
     DATETIME = "DATETIME"
 
 
+class SqlParameterScalarTypes:
+    """Supported scalar SQL query parameter types as type objects."""
+
+    STRING = ScalarQueryParameterType("STRING")
+    BYTES = ScalarQueryParameterType("BYTES")
+    INTEGER = ScalarQueryParameterType("INT64")
+    INT64 = ScalarQueryParameterType("INT64")
+    FLOAT = ScalarQueryParameterType("FLOAT64")
+    FLOAT64 = ScalarQueryParameterType("FLOAT64")
+    NUMERIC = ScalarQueryParameterType("NUMERIC")
+    BIGNUMERIC = ScalarQueryParameterType("BIGNUMERIC")
+    DECIMAL = ScalarQueryParameterType("NUMERIC")
+    BIGDECIMAL = ScalarQueryParameterType("BIGNUMERIC")
+    BOOLEAN = ScalarQueryParameterType("BOOL")
+    BOOL = ScalarQueryParameterType("BOOL")
+    GEOGRAPHY = ScalarQueryParameterType("GEOGRAPHY")
+    TIMESTAMP = ScalarQueryParameterType("TIMESTAMP")
+    DATE = ScalarQueryParameterType("DATE")
+    TIME = ScalarQueryParameterType("TIME")
+    DATETIME = ScalarQueryParameterType("DATETIME")
+
+
 class WriteDisposition(object):
     """Specifies the action that occurs if destination table already exists.
 
